@@ -18,7 +18,7 @@ from plotly.offline import iplot
 
 # Project imports
 from groundhog.general.plotting import plot_with_log, GROUNDHOG_PLOTTING_CONFIG
-from groundhog.general.parameter_mapping import offsets
+from groundhog.general.parameter_mapping import offsets, latlon_distance
 
 
 class SoilProfile(pd.DataFrame):
@@ -42,6 +42,23 @@ class SoilProfile(pd.DataFrame):
         self.depth_from_col = "%s from [%s]" % (name, unit)
         self.depth_to_col = "%s to [%s]" % (name, unit)
 
+    def convert_depth_reference(self, newname="Depth", newunit='m', multiplier=1):
+        """
+        Converts the depth reference for a soil profile between one set of units (e.g. ft) to another (e.g. m)
+        :param newname: New name for the depth reference (default='Depth')
+        :param newunit: Name of the new unit (default=m)
+        :param multiplier: Multiplier to go from old to new depth unit (e.g. 0.3 to go from ft to m)
+        :return:
+        """
+        self[self.depth_from_col] = self[self.depth_from_col] * multiplier
+        self[self.depth_to_col] = self[self.depth_to_col] * multiplier
+        self.rename(columns={
+            self.depth_from_col: "%s from [%s]" % (newname, newunit),
+            self.depth_to_col: "%s to [%s]" % (newname, newunit)
+        }, inplace=True)
+        self.depth_from_col = "%s from [%s]" % (newname, newunit)
+        self.depth_to_col = "%s to [%s]" % (newname, newunit)
+
     def check_profile(self):
         """
         Check if the SoilProfile meets the requirements for calculations
@@ -303,7 +320,7 @@ class SoilProfile(pd.DataFrame):
         x = np.insert(second_array_x, np.arange(len(first_array_x)), first_array_x)
         return z, x
 
-    def map_soilprofile(self, nodalcoords, target_depthkey='z [m]', invert_sign=False, offset=0):
+    def map_soilprofile(self, nodalcoords, target_depthkey='z [m]', keys_to_map=None, invert_sign=False, offset=0):
         """
         Maps the soilprofile to a grid. The depth coordinates to the grid are specified
         in a list or Numpy array (``nodalcoords``).
@@ -312,6 +329,7 @@ class SoilProfile(pd.DataFrame):
         All soil parameters are interpolated onto this grid.
         :param nodalcoords: List or Numpy array with the nodal coordinates of the grid
         :param target_depthkey: Name of the depth key in the resulting dataframe
+        :param keys_to_map: List with the soilprofile keys to map
         :param invert_sign: Boolean determining whether to invert the sign after interpolation
         :param offset: Offset by which the depth is shifted (added to the depth after sign conversion)
         :return: Returns a dataframe with the full grid with soil parameters
@@ -332,15 +350,33 @@ class SoilProfile(pd.DataFrame):
         else:
             raise ValueError("Nodal coordinates are not ascending. Please check ")
 
+        if keys_to_map is None:
+            # Map all keys
+            string_params = self.string_soil_parameters()
+            numerical_params = self.numerical_soil_parameters()
+        else:
+            # Map only specific keys
+            string_params = []
+            numerical_params = []
+            for _key in keys_to_map:
+                if _key in self.string_soil_parameters():
+                    string_params.append(_key)
+                elif _key in self.numerical_soil_parameters():
+                    numerical_params.append(_key)
+                else:
+                    raise ValueError(
+                        "Key %s is not present in the numerical or string parameters of the soil profile" % _key)
+
         # 3. Map string parameters
-        for _param in self.string_soil_parameters():
+        for _param in string_params:
             target_df[_param] = list(map(lambda _z: self[
                 (self[self.depth_from_col] <= _z) & (self[self.depth_to_col] >= _z)][_param].iloc[-1],
                                       target_df[target_depthkey]))
 
         # 4. Map numerical parameters
-        for _param in self.numerical_soil_parameters():
+        for _param in numerical_params:
             z, x = self.soilparameter_series(_param)
+
             target_df[_param] = np.interp(target_df[target_depthkey], z, x)
 
         # 5. Convert sign and apply offset
@@ -421,6 +457,8 @@ class SoilProfile(pd.DataFrame):
             soildata=self,
             xtitles=x_titles,
             ztitle=z_title,
+            depth_from_key=self.depth_from_col,
+            depth_to_key=self.depth_to_col,
             **kwargs
         )
 
@@ -702,11 +740,12 @@ class SoilProfile(pd.DataFrame):
                 self.loc[i, outputparameter.replace(' [', ' from [')] + \
                 row[parameter] * (row[self.depth_to_col] - row[self.depth_from_col])
 
-    def calculate_overburden(self, waterlevel=0, waterunitweight=10, totalunitweightcolumn="Total unit weight [kN/m3]",
+    def calculate_overburden(self, waterlevel=0, waterunitweight=10, initial_vertical_total_stress=0,
+                             totalunitweightcolumn="Total unit weight [kN/m3]",
                              effectiveunitweightcolumn="Effective unit weight [kN/m3]",
                              waterunitweightcolumn="Water unit weight [kN/m3]",
-                             totalverticalstresscolumn="Total vertical stress [kPa]",
-                             effectiveverticalstresscolumn="Effective vertical stress [kPa]",
+                             totalverticalstresscolumn="Vertical total stress [kPa]",
+                             effectiveverticalstresscolumn="Vertical effective stress [kPa]",
                              hydrostaticpressurecolumn="Hydrostatic pressure [kPa]"):
         """
         Calculates the overburden pressure (total and effective) for a ``SoilProfile`` object.
@@ -719,13 +758,16 @@ class SoilProfile(pd.DataFrame):
         Note that vertical stress calculations in other units are possible, but the water unit weight then needs
         to be specified in consistent units.
 
+        An initial value can be added to the total vertical stress to simulate the effect of e.g. surcharging.
+
         :param waterlevel: Water level [m] (default 0m)
         :param waterunitweight: Unit weight of the pore water [kN/m3] (default=10kN/m3)
+        :param initial_vertical_total_stress: Initial value of vertical total stress [kPa] (default=0kPa)
         :param totalunitweightcolumn: Column name containing total unit weights (default='Total unit weight [kN/m3]'
         :param waterunitweightcolumn: Output column with the effective unit weight (default='Effective unit weight [kN/m3]')
         :param waterunitweightcolumn: Output column with the water unit weight (default='Water unit weight [kN/m3]')
         :param totalverticalstresscolumn: Total vertical stress column name (default='Total vertical stress [kPa]')
-        :param effectiveverticalstresscolumn: Effective vertical stress column name (default='Effective vertical stress [kPa]')
+        :param effectiveverticalstresscolumn: Effective vertical stress column name (default='Vertical effective stress [kPa]')
         :param hydrostaticpressurecolumn: Hydrostatic pressure column name (default='Hydrostatic pressure [kPa]')
         :return: Adds the column names from ``totalverticalstresscolumn``, ``effectiveverticalstresscolumn`` and ``hydrostaticpressurecolumn`` to the ``SoilProfile`` object
         """
@@ -766,10 +808,12 @@ class SoilProfile(pd.DataFrame):
         self.depth_integration(parameter=waterunitweightcolumn, outputparameter=hydrostaticpressurecolumn)
 
         # Calculate effective vertical stress
-        self.depth_integration(parameter=effectiveunitweightcolumn, outputparameter=effectiveverticalstresscolumn)
+        self.depth_integration(parameter=effectiveunitweightcolumn, outputparameter=effectiveverticalstresscolumn,
+                               start_value=initial_vertical_total_stress)
 
         # Calculate total vertical stress
-        self.depth_integration(parameter=totalunitweightcolumn, outputparameter=totalverticalstresscolumn)
+        self.depth_integration(parameter=totalunitweightcolumn, outputparameter=totalverticalstresscolumn,
+                               start_value=initial_vertical_total_stress)
 
 
     def applyfunction(self, function, resultkey, outputkey, parametermapping=dict(), **kwargs):
@@ -815,8 +859,51 @@ class SoilProfile(pd.DataFrame):
                     self.loc[i, outputkey] = \
                         function(**{**function_dict, **kwargs})[resultkey]
 
+    def parameter_at_depth(self, depth, parameter, shallowest=True):
+        """
+        Calculates the value for one of the ``SoilProfile`` parameters at a selected depth
+        :param depth: Selected depth [m]
+        :param parameter: String or Numerical soil parameter (linear interpolation for linearly varying parameters)
+        :param shallowest: Boolean determining whether at a layer interface, the shallowest value needs to be used (default=True).
+        :return: The value of the parameter at the selected depth
+        """
+        if depth > self.max_depth:
+            raise ValueError("Selected depth is greater than the maximum depth of the soil profile")
 
-def read_excel(path, title='', depth_key='Depth', unit='m', column_mapping={}, **kwargs):
+        if depth < self.min_depth:
+            raise ValueError("Selected depth is lower than the minimum depth of the soil profile")
+
+        if (not parameter in self.numerical_soil_parameters()) and \
+                (not parameter in self.string_soil_parameters()):
+            raise ValueError("Selected parameter is not in the list of available parameters")
+
+        _layers = self[
+             (self[self.depth_from_col] <= depth) &
+             (self[self.depth_to_col] >= depth)
+        ]
+
+        if shallowest:
+            _selected_layer = _layers.iloc[0]
+        else:
+            _selected_layer = _layers.iloc[-1]
+
+        if parameter in self.numerical_soil_parameters():
+            if self.check_linear_variation(parameter):
+                # Linear interpolation for a linearly varying parameter
+                return np.interp(
+                    depth,
+                    [_selected_layer[self.depth_from_col], _selected_layer[self.depth_to_col]],
+                    [_selected_layer[parameter.replace(' [', ' from [')],
+                     _selected_layer[parameter.replace(' [', ' to [')]]
+                )
+            else:
+                # Constant value for a parameter with constant value in the layer
+                return _selected_layer[parameter]
+        else:
+            # String in the selected layer
+            return _selected_layer[parameter]
+
+def read_excel(path, title='', depth_key='Depth', unit='m', column_mapping={}, depth_multiplier=1, **kwargs):
     """
     The method to read from Excel needs to be redefined for SoilProfile objects.
     The method allows for different depth keys (using the 'depth_key' and 'unit' keyword arguments
@@ -851,23 +938,28 @@ def profile_from_dataframe(df, title='', depth_key='Depth', unit='m', column_map
 
 
 def plot_fence_diagram(
-    profiles=[],
+    profiles=[], latlon=False,
     option='name', start=None, end=None, band=1000, extend_profile=False,
+    soiltypekey="Soil type",
+    plotmap=False,
     fillcolordict={'SAND': 'yellow', 'CLAY': 'brown', 'SILT': 'green', 'ROCK': 'grey'},
     opacity=1, logwidth=1, distance_unit='m', return_layers=False,
     showfig=True, xaxis_layout=None, yaxis_layout=None, general_layout=None,
     show_annotations=True):
     """
-    Creates a longitudinal profile along selected CPTs. A line is drawn from the first (smallest distance from origin)
-    to the last location (greatest distance from origin) and the plot of the selected parameter (``prop``) vs depth
+    Creates a longitudinal profile along selected soil profiles. A line is drawn from the first (smallest distance from origin)
+    to the last location (greatest distance from origin) and the plot of the mini-logs with soil types
     is projected onto this line.
 
     :param profiles: List with SoilProfile objects for which a log needs to be plotted
+    :param latlon: Boolean defining whether coordinates are specified in latitude and longitude (defaulte=False). If this is the case, offsets are calculated using the ``pyproj`` package
     :param option: Determines whether soil profile names (``option='name'``) or tuples with coordinates (``option='coords'``) are used for the ``start`` and ``end`` arguments
-    :param start: Soil profile name for the starting point or tuple of coordinates. If a CPT name is used, the selected CPT must be contained in ``cpts``.
-    :param end: CPT name for the end point or tuple of coordinates. If a CPT name is used, the selected CPT must be contained in ``cpts``.
-    :param band: Offset from the line connecting start and end points in which CPT are considered for plotting (default=1000m)
+    :param start: Soil profile name for the starting point or tuple of coordinates. If a SoilProfile name is used, the selected SoilProfile must be contained in ``profiles``.
+    :param end: Soil profile name for the end point or tuple of coordinates. If a SoilProfile name is used, the selected SoilProfile must be contained in ``profiles``.
+    :param band: Offset from the line connecting start and end points in which soil profiles are considered for plotting (default=1000m)
     :param extend_profile: Boolean determining whether the profile needs to be extended beyond the start and end points (default=False)
+    :param plotmap: Boolean determining whether a map of locations needs to be plotted next to the profile (default=False)
+    :param soiltypekey: Key for the soil type in the dataframes with layering (default="Soil type")
     :param fillcolordict: Dictionary with fill colours (default yellow for 'SAND', brown from 'CLAY' and grey for 'ROCK')
     :param opacity: Opacity of the layers (default = 1 for non-transparent behaviour)
     :param logwidth: Width of the soil logs as an absolute value (default = 1)
@@ -932,13 +1024,17 @@ def plot_fence_diagram(
             profile_df.loc[i, "Behind end"] = False
         elif row['X'] == end_point[0] and row['Y'] == end_point[1]:
             profile_df.loc[i, "Offset"] = 0
-            profile_df.loc[i, "Projected offset"] = np.sqrt(
-                (start_point[0] - end_point[0]) ** 2 +
-                (start_point[1] - end_point[1]) ** 2)
+            if latlon:
+                profile_df.loc[i, "Projected offset"] = latlon_distance(
+                    start_point[0], start_point[1], end_point[0], end_point[1])
+            else:
+                profile_df.loc[i, "Projected offset"] = np.sqrt(
+                    (start_point[0] - end_point[0]) ** 2 +
+                    (start_point[1] - end_point[1]) ** 2)
             profile_df.loc[i, "Before start"] = False
             profile_df.loc[i, "Behind end"] = False
         else:
-            result = offsets(start_point, end_point, (row['X'], row['Y']))
+            result = offsets(start_point, end_point, (row['X'], row['Y']), latlon=latlon)
             profile_df.loc[i, "Offset"] = result['offset to line']
             profile_df.loc[i, "Projected offset"] = result['offset to start projected']
             profile_df.loc[i, "Before start"] = result['before start']
@@ -962,7 +1058,7 @@ def plot_fence_diagram(
     for i, row in selected_profiles.iterrows():
 
         for j, _layer in row["Soil profiles"].iterrows():
-            _fillcolor = fillcolordict[_layer['Soil type']]
+            _fillcolor = fillcolordict[_layer[soiltypekey]]
             _y0 = row['Z'] - _layer['Depth from [m]']
             _y1 = row['Z'] - _layer['Depth to [m]']
             _x0 = row['Projected offset'] - 0.5 * logwidth
@@ -1021,7 +1117,11 @@ def plot_fence_diagram(
     if return_layers:
         return _layers, _annotations, _backbone_traces, _soilcolors
     else:
-        fig = subplots.make_subplots(rows=1, cols=1, print_grid=False)
+        if plotmap:
+            fig = subplots.make_subplots(rows=1, cols=2, print_grid=False, column_widths=[0.7, 0.3],
+                                         specs=[[{'type': 'xy'}, {'type': 'mapbox'},]])
+        else:
+            fig = subplots.make_subplots(rows=1, cols=1, print_grid=False)
 
         for _trace in _backbone_traces:
             fig.append_trace(_trace, 1, 1)
@@ -1029,6 +1129,17 @@ def plot_fence_diagram(
         for _trace in _soilcolors:
             fig.append_trace(_trace, 1, 1)
 
+        if plotmap:
+            mapbox_points = go.Scattermapbox(
+                lat=y_coords, lon=x_coords, showlegend=False,
+                mode='markers', name='Locations', hovertext=profile_names)
+            fig.append_trace(mapbox_points, 1, 2)
+            mapbox_profileline = go.Scattermapbox(
+                lat=[start_point[1], end_point[1]],
+                lon=[start_point[0], end_point[0]],
+                showlegend=False, mode='lines', name='Profile')
+            fig.append_trace(mapbox_profileline, 1, 2)
+
         if xaxis_layout is None:
             fig['layout']['xaxis1'].update(title='Projected distance [%s]' % (distance_unit))
         else:
@@ -1048,6 +1159,12 @@ def plot_fence_diagram(
 
         fig['layout'].update(shapes=_layers)
 
+        if plotmap:
+            fig.update_layout(
+                mapbox_style='open-street-map', mapbox_zoom=10,
+                mapbox_center={'lat': profile_df['Y'].mean(), 'lon': profile_df['X'].mean()}
+            )
+
         if show_annotations:
             fig['layout'].update(annotations=_annotations)
         if showfig:
